﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;

using System.IO;
using System.Media;
using System.Threading;
using System.IO;

using Microsoft.Kinect;


namespace MotionDetector
{
    /// <summary>
    /// Interaction logic for MainWindow.xaml
    /// </summary>
    public partial class MainWindow : Window
    {

        KinectSensor myKinect;

        SoundPlayer alarmPlayer;

        public MainWindow()
        {
            InitializeComponent();
        }

        private void Window_Loaded(object sender, RoutedEventArgs e)
        {
            if (KinectSensor.KinectSensors.Count == 0)
            {
                MessageBox.Show("No Kinects detected", "Camera Viewer");
                Application.Current.Shutdown();
            }

            try
            {
                myKinect = KinectSensor.KinectSensors[0];

                myKinect.ColorStream.Enable();

                myKinect.Start();
            }
            catch
            {
                MessageBox.Show("Kinect initialise failed", "Camera Viewer");
                Application.Current.Shutdown();
            }

            updateVideoThread = new Thread(new ThreadStart(videoDisplay));

            updateVideoThread.Start();

            alarmPlayer = new SoundPlayer("ding.wav");

        }

        WriteableBitmap colorImageBitmap = null;
        byte[] currentImageBytes = null;
        byte[] previousFrameBytes = null;

        byte greenChangeThreshold = 20;
        byte blueChangeThreshold = 20;
        byte redChangeThreshold = 20;
        int differences;
        int alarmThreshold;

        unsafe int differenceCount(byte[] videoImageBytes)
        {
            int differences = 0;

            // Make the previous frame buffer if we need to

            if (previousFrameBytes == null)
            {
                previousFrameBytes = new byte[videoImageBytes.Length];
                Buffer.BlockCopy(videoImageBytes, 0, previousFrameBytes, 0, videoImageBytes.Length);
                // return  0 as we don't have any differences yet
                return 0;
            }

            byte green, blue, red;
            byte previousGreen, previousBlue, previousRed;
            int greenDiff, blueDiff, redDiff;

            fixed (byte* imageBase = videoImageBytes, previousBase = previousFrameBytes)
            {
                byte* imagePosition = imageBase;
                byte* previousPosition = previousBase;
                byte* redPos, greenPos, bluePos;

                byte* imageEnd = imageBase + videoImageBytes.Length;

                while (imagePosition != imageEnd)
                {
                    // Get the new values from the image

                    green = *imagePosition;
                    greenPos = imagePosition;
                    imagePosition++;

                    blue = *imagePosition;
                    bluePos = imagePosition;
                    imagePosition++;

                    red = *imagePosition;
                    redPos = imagePosition;
                    imagePosition++;

                    // Skip past the alpha in the original image
                    imagePosition++;

                    // Get the previous values and then
                    // store the latest values in place of them

                    previousGreen = *previousPosition;
                    *previousPosition = green;
                    previousPosition++;

                    previousBlue = *previousPosition;
                    *previousPosition = blue;
                    previousPosition++;

                    previousRed = *previousPosition;
                    *previousPosition = red;
                    previousPosition++;

                    // Skip past the alpha in the previous image
                    previousPosition++;

                    if (green > previousGreen)
                        greenDiff = green - previousGreen;
                    else
                        greenDiff = previousGreen - green;

                    if (blue > previousBlue)
                        blueDiff = blue - previousBlue;
                    else
                        blueDiff = previousBlue - blue;

                    if (red > previousRed)
                        redDiff = red - previousRed;
                    else
                        redDiff = previousRed - red;

                    if (greenDiff > greenChangeThreshold || blueDiff > blueChangeThreshold || redDiff > redChangeThreshold)
                    {
                        *greenPos = 255;
                        *bluePos = 255;
                        *redPos = 255;
                        differences++;
                    }
                }
            }

            return differences;
        }

        bool displayActive = true;

        Thread updateVideoThread;

        void videoDisplay()
        {
            while (displayActive)
            {
                using (ColorImageFrame colorFrame = myKinect.ColorStream.OpenNextFrame(10))
                {
                    if (colorFrame == null) continue;

                    if (currentImageBytes == null)
                        currentImageBytes = new byte[colorFrame.PixelDataLength];

                    colorFrame.CopyPixelDataTo(currentImageBytes);

                    differences = differenceCount(currentImageBytes);

                    Dispatcher.Invoke(new Action(() => updateDisplay()));
                }
            }

            // when we get here the program is ending – stop the sensor
            myKinect.Stop();
        }

        SolidColorBrush OKBrush = new SolidColorBrush(Colors.Green);
        SolidColorBrush AlarmBrush = new SolidColorBrush(Colors.Red);

        void updateDisplay()
        {

            differenceValue.Content = differences.ToString();
            thresholdValue.Content = alarmThreshold.ToString();
            greenValue.Content = greenChangeThreshold.ToString();
            blueValue.Content = blueChangeThreshold.ToString();
            redValue.Content = redChangeThreshold.ToString();

            if (differences > alarmThreshold)
            {
                if (alarmLabel.Background != AlarmBrush)
                {
                    alarmPlayer.Play();
                    alarmLabel.Background = AlarmBrush;
                }
            }
            else
            {
                alarmLabel.Background = OKBrush;
            }

            if (colorImageBitmap == null)
            {
                this.colorImageBitmap = new WriteableBitmap(
                    640,
                    480,
                    96,  // DpiX
                    96,  // DpiY
                    PixelFormats.Bgr32,
                    null);
                kinectVideo.Source = colorImageBitmap;
            }

            colorImageBitmap.WritePixels(
                new Int32Rect(0, 0, 640, 480),
                currentImageBytes, // video data
                640 * 4, // stride,
                0   // offset into the array - start at 0
                );
        }

        private void redSlider_ValueChanged(object sender, RoutedPropertyChangedEventArgs<double> e)
        {
            redChangeThreshold = (byte)redSlider.Value;
        }

        private void greenSlider_ValueChanged(object sender, RoutedPropertyChangedEventArgs<double> e)
        {
            greenChangeThreshold = (byte)greenSlider.Value;
        }

        private void blueSlider_ValueChanged(object sender, RoutedPropertyChangedEventArgs<double> e)
        {
            blueChangeThreshold = (byte)blueSlider.Value;
        }

        private void thresholdSlider_ValueChanged(object sender, RoutedPropertyChangedEventArgs<double> e)
        {
            alarmThreshold = (int)thresholdSlider.Value;
        }

        private void Window_Closing(object sender, System.ComponentModel.CancelEventArgs e)
        {
            displayActive = false;
        }
    }
}
