﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;

using Microsoft.Kinect;
using System.IO;
using System.Threading;
using System.Media;


namespace MotionDetector
{
    /// <summary>
    /// Interaction logic for MainWindow.xaml
    /// </summary>
    public partial class MainWindow : Window
    {

        KinectSensor myKinect;

        string alarmFileName = "alarm.wav";

        public MainWindow()
        {
            InitializeComponent();
        }

        private void Window_Loaded(object sender, RoutedEventArgs e)
        {

            if (KinectSensor.KinectSensors.Count == 0)
            {
                MessageBox.Show("No Kinects detected", "Camera Viewer");
                Application.Current.Shutdown();
            }

            try
            {
                myKinect = KinectSensor.KinectSensors[0];

                myKinect.ColorStream.Enable();

                myKinect.Start();
            }
            catch
            {
                MessageBox.Show("Kinect initialise failed", "Camera Viewer");
                Application.Current.Shutdown();
            }

            updateVideoThread = new Thread(new ThreadStart(videoDisplay));

            updateVideoThread.Start();

        }

        WriteableBitmap colorImageBitmap = null;
        byte[] currentImageBytes = null;
        byte[] previousFrameBytes = null;

        int whiteChangeThreshold = 60;
        int differences;
        int alarmThreshold;

        unsafe int differenceCount(byte[] videoImageBytes)
        {
            int differences = 0;

            // Make the previous frame buffer if we need to

            if (previousFrameBytes == null)
            {
                previousFrameBytes = new byte[videoImageBytes.Length];
                Buffer.BlockCopy(videoImageBytes, 0, previousFrameBytes, 0, videoImageBytes.Length);
                // return  0 as we don't have any differences yet
                return 0;
            }

            byte green, blue, red;
            byte previousGreen, previousBlue, previousRed;

            fixed (byte* imageBase = videoImageBytes, previousBase = previousFrameBytes)
            {
                byte* imagePosition = imageBase;
                byte* previousPosition = previousBase;
                byte* redPos, greenPos, bluePos;

                byte* imageEnd = imageBase + videoImageBytes.Length;

                while (imagePosition != imageEnd)
                {
                    // Get the new values from the image

                    green = *imagePosition;
                    greenPos = imagePosition;
                    imagePosition++;

                    blue = *imagePosition;
                    bluePos = imagePosition;
                    imagePosition++;

                    red = *imagePosition;
                    redPos = imagePosition;
                    imagePosition++;

                    // Skip past the alpha in the original image
                    imagePosition++;

                    // Get the previous values and then
                    // store the latest values in place of them

                    previousGreen = *previousPosition;
                    *previousPosition = green;
                    previousPosition++;

                    previousBlue = *previousPosition;
                    *previousPosition = blue;
                    previousPosition++;

                    previousRed = *previousPosition;
                    *previousPosition = red;
                    previousPosition++;

                    // Skip past the alpha in the previous image
                    previousPosition++;

                    int whiteLevel = red + blue + green;

                    // Convert the image to black and white as well
                    byte whiteByte = (byte)(whiteLevel / 3);

                    int previousWiteLevel = previousRed + previousBlue + previousGreen;
                    int whiteDiff;

                    if (whiteLevel > previousWiteLevel)
                        whiteDiff = whiteLevel - previousWiteLevel;
                    else
                        whiteDiff = previousWiteLevel - whiteLevel;

                    if (whiteDiff > whiteChangeThreshold)
                    {
                        *greenPos = 255;
                        *bluePos = 255;
                        *redPos = 255;
                        differences++;
                    }
                    else
                    {
                        *greenPos = whiteByte;
                        *bluePos = whiteByte;
                        *redPos = whiteByte;
                    }
                }
            }

            return differences;
        }

        bool displayActive = true;

        Thread updateVideoThread;

        int picturesToTake = 0;

        void videoDisplay()
        {
            while (displayActive)
            {
                using (ColorImageFrame colorFrame = myKinect.ColorStream.OpenNextFrame(10))
                {
                    if (colorFrame == null) continue;

                    if (currentImageBytes == null)
                        currentImageBytes = new byte[colorFrame.PixelDataLength];

                    colorFrame.CopyPixelDataTo(currentImageBytes);

                    differences = differenceCount(currentImageBytes);

                    Dispatcher.Invoke(new Action(() => updateDisplay()));
                }
            }

            // when we get here the program is ending – stop the sensor

            myKinect.Stop();
        }

        SolidColorBrush OKBrush = new SolidColorBrush(Colors.Green);
        SolidColorBrush AlarmBrush = new SolidColorBrush(Colors.Red);

        void updateDisplay()
        {

            differenceValue.Content = differences.ToString();
            thresholdValue.Content = alarmThreshold.ToString();
            whiteValue.Content = whiteChangeThreshold.ToString();
            if (differences > alarmThreshold)
            {
                if (!alarmSoundActive && !audioCaptureActive)
                {
                    alarmPlayer.Source = new Uri(alarmFileName, UriKind.RelativeOrAbsolute);
                    alarmPlayer.LoadedBehavior = MediaState.Play;
                    alarmPlayer.UnloadedBehavior = MediaState.Close;
                    alarmSoundActive = true;
                    alarmLabel.Background = AlarmBrush;
                    picturesToTake = 5;
                }
            }
            else
            {
                alarmLabel.Background = OKBrush;
            }

            if (colorImageBitmap == null)
            {
                this.colorImageBitmap = new WriteableBitmap(
                    640,
                    480,
                    96,  // DpiX
                    96,  // DpiY
                    PixelFormats.Bgr32,
                    null);
                kinectVideo.Source = colorImageBitmap;
            }

            colorImageBitmap.WritePixels(
                new Int32Rect(0, 0, 640, 480),
                currentImageBytes, // video data
                640 * 4, // stride,
                0   // offset into the array - start at 0
                );

            if (picturesToTake > 0)
            {
                saveFrame(previousFrameBytes, 640, 480);
                picturesToTake--;
            }
        }

        int frameNo = 1;

        void saveFrame(byte[] videoImage, int width, int height)
        {
            string message = "";
            if (fileNameTextBox.Text == "")
            {
                message = "No filename set";
            }
            else
            {
                try
                {
                    BitmapSource pictureBitmap = null;

                    pictureBitmap = BitmapSource.Create(
                        width, height, 96, 96, PixelFormats.Bgr32, null,
                        videoImage, width * 4);

                    // Save image 
                    string destDir = System.IO.Path.GetDirectoryName(fileNameTextBox.Text);
                    string destFile = System.IO.Path.GetFileNameWithoutExtension(fileNameTextBox.Text);
                    string fileName = destDir + @"\" + destFile + frameNo.ToString() + ".jpg";

                    using (FileStream stream = new FileStream(fileName, FileMode.Create))
                    {
                        JpegBitmapEncoder encoder = new JpegBitmapEncoder();
                        encoder.Frames.Add(BitmapFrame.Create(pictureBitmap));
                        encoder.Save(stream);
                    }
                    frameNo++;
                    message = fileName + " saved OK";
                }
                catch
                {
                    message = "Save failed";
                }
            }

            Dispatcher.Invoke(new Action(() => { fileSaveStatusLabel.Content = message; }));
        }

        bool alarmSoundActive = false;

        private void whiteSlider_ValueChanged(object sender, RoutedPropertyChangedEventArgs<double> e)
        {
            whiteChangeThreshold = (int)whiteSlider.Value;
        }

        private void thresholdSlider_ValueChanged(object sender, RoutedPropertyChangedEventArgs<double> e)
        {
            alarmThreshold = (int)thresholdSlider.Value;
        }

        #region Audio File save

        #region WAV file header code

        struct WAVEFORMATEX
        {
            public ushort wFormatTag;
            public ushort nChannels;
            public uint nSamplesPerSec;
            public uint nAvgBytesPerSec;
            public ushort nBlockAlign;
            public ushort wBitsPerSample;
            public ushort cbSize;
        }

        /// <summary>
        /// A bare bones WAV file header writer
        /// </summary>        
        static void WriteWavHeader(Stream stream, int dataLength)
        {
            //We need to use a memory stream because the BinaryWriter will close the underlying stream when it is closed
            using (MemoryStream memStream = new MemoryStream(64))
            {
                int cbFormat = 18; //sizeof(WAVEFORMATEX)
                WAVEFORMATEX format = new WAVEFORMATEX()
                {
                    wFormatTag = 1,
                    nChannels = 1,
                    nSamplesPerSec = 16000,
                    nAvgBytesPerSec = 32000,
                    nBlockAlign = 2,
                    wBitsPerSample = 16,
                    cbSize = 0
                };

                using (var bw = new BinaryWriter(memStream))
                {
                    //RIFF header
                    WriteString(memStream, "RIFF");
                    bw.Write(dataLength + cbFormat + 4); //File size - 8
                    WriteString(memStream, "WAVE");
                    WriteString(memStream, "fmt ");
                    bw.Write(cbFormat);

                    //WAVEFORMATEX
                    bw.Write(format.wFormatTag);
                    bw.Write(format.nChannels);
                    bw.Write(format.nSamplesPerSec);
                    bw.Write(format.nAvgBytesPerSec);
                    bw.Write(format.nBlockAlign);
                    bw.Write(format.wBitsPerSample);
                    bw.Write(format.cbSize);

                    //data header
                    WriteString(memStream, "data");
                    bw.Write(dataLength);
                    memStream.WriteTo(stream);
                }
            }
        }

        static void WriteString(Stream stream, string s)
        {
            byte[] bytes = Encoding.ASCII.GetBytes(s);
            stream.Write(bytes, 0, bytes.Length);
        }

        #endregion

        static int recordTimeInSecs = 4;
        static int recordBufferSize = recordTimeInSecs * 2 * 16000;
        byte[] soundSampleBuffer = new byte[recordBufferSize];
        bool audioCaptureActive = false;

        void showAlarmSoundStatus(string message)
        {
            Dispatcher.Invoke(new Action(() => { alarmSoundStatusLabel.Content = message; }));
        }

        void captureAudio()
        {
            if (alarmSoundActive)
            {
                showAlarmSoundStatus("Cannot record - alarm sound active");
                return;
            }

            showAlarmSoundStatus("Recording");

            try
            {
                // Create a Kinect audio source and a sound input stream
                Stream kinectAudioStream = myKinect.AudioSource.Start();
                // Get the sound sample
                kinectAudioStream.Read(soundSampleBuffer, 0, soundSampleBuffer.Length);

                // Get the output stream
                using (var fileStream = new FileStream(alarmFileName, FileMode.Create))
                {
                    WriteWavHeader(fileStream, recordBufferSize);
                    fileStream.Write(soundSampleBuffer, 0, soundSampleBuffer.Length);
                }

                showAlarmSoundStatus("Recorded OK");
            }
            catch
            {
                showAlarmSoundStatus("Record Failed");
                return;
            }
            finally
            {
                audioCaptureActive = false;
            }
        }

        #endregion

        private void fileNameButton_Click(object sender, RoutedEventArgs e)
        {
            // Configure save file dialog box
            Microsoft.Win32.SaveFileDialog dlg = new Microsoft.Win32.SaveFileDialog();
            dlg.FileName = "Shot"; // Default file name
            dlg.DefaultExt = ".jpg"; // Default file extension
            dlg.Filter = "Pictures (.jpg)|*.jpg"; // Filter files by extension

            // Process save file dialog box results
            if (dlg.ShowDialog() == true)
            {
                // set the filename
                fileNameTextBox.Text = dlg.FileName;
            }
        }

        private void recordSoundButton_Click(object sender, RoutedEventArgs e)
        {
            if (audioCaptureActive) return;

            audioCaptureActive = true;

            Thread audioThread = new Thread(new ThreadStart(captureAudio));

            audioThread.Priority = ThreadPriority.Highest;

            audioThread.Start();
        }

        private void alarmPlayer_MediaEnded(object sender, RoutedEventArgs e)
        {
            alarmSoundActive = false;
            alarmPlayer.Source = null;
        }
    }
}
