﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;

using System.Media;
using Microsoft.Kinect;

namespace KinectKissDetector
{
    /// <summary>
    /// Interaction logic for MainWindow.xaml
    /// </summary>
    public partial class MainWindow : Window
    {
        SoundPlayer alarmPlayer;

        KinectSensor myKinect;

        public MainWindow()
        {
            InitializeComponent();
        }

        private void Window_Loaded(object sender, RoutedEventArgs e)
        {
            // Check to see if a Kinect is available
            if (KinectSensor.KinectSensors.Count == 0)
            {
                MessageBox.Show("No Kinects detected", "Camera Viewer");
                Application.Current.Shutdown();
            }

            // Get the first Kinect on the computer
            myKinect = KinectSensor.KinectSensors[0];

            // Start the Kinect running and select the depth camera
            try
            {
                myKinect.SkeletonStream.Enable();
                myKinect.Start();
            }
            catch
            {
                MessageBox.Show("Kinect initialise failed", "Camera Viewer");
                Application.Current.Shutdown();
            }

            // connect a handler to the event that fires when new frames are available

            myKinect.SkeletonFrameReady += new EventHandler<SkeletonFrameReadyEventArgs>(myKinect_SkeletonFrameReady);

            // Load the sound

            alarmPlayer = new SoundPlayer("alarm.wav");

        }

        void addLine(Joint j1, Joint j2, Brush skeletonBrush)
        {
            Line boneLine = new Line();
            boneLine.Stroke = skeletonBrush;
            boneLine.StrokeThickness = 5;

            ColorImagePoint j1P = myKinect.MapSkeletonPointToColor(j1.Position,
                                      ColorImageFormat.RgbResolution640x480Fps30);
            boneLine.X1 = j1P.X;
            boneLine.Y1 = j1P.Y;

            ColorImagePoint j2P = myKinect.MapSkeletonPointToColor(j2.Position,
                                      ColorImageFormat.RgbResolution640x480Fps30);
            boneLine.X2 = j2P.X;
            boneLine.Y2 = j2P.Y;


            skeletonCanvas.Children.Add(boneLine);
        }

        void drawSkeleton(Skeleton skeleton, Brush drawBrush)
        {

            // Spine
            addLine(skeleton.Joints[JointType.Head], skeleton.Joints[JointType.ShoulderCenter], drawBrush);
            addLine(skeleton.Joints[JointType.ShoulderCenter], skeleton.Joints[JointType.Spine], drawBrush);

            // Left leg
            addLine(skeleton.Joints[JointType.Spine], skeleton.Joints[JointType.HipCenter], drawBrush);
            addLine(skeleton.Joints[JointType.HipCenter], skeleton.Joints[JointType.HipLeft], drawBrush);
            addLine(skeleton.Joints[JointType.HipLeft], skeleton.Joints[JointType.KneeLeft], drawBrush);
            addLine(skeleton.Joints[JointType.KneeLeft], skeleton.Joints[JointType.AnkleLeft], drawBrush);
            addLine(skeleton.Joints[JointType.AnkleLeft], skeleton.Joints[JointType.FootLeft], drawBrush);

            // Right leg
            addLine(skeleton.Joints[JointType.HipCenter], skeleton.Joints[JointType.HipRight], drawBrush);
            addLine(skeleton.Joints[JointType.HipRight], skeleton.Joints[JointType.KneeRight], drawBrush);
            addLine(skeleton.Joints[JointType.KneeRight], skeleton.Joints[JointType.AnkleRight], drawBrush);
            addLine(skeleton.Joints[JointType.AnkleRight], skeleton.Joints[JointType.FootRight], drawBrush);

            // Left arm
            addLine(skeleton.Joints[JointType.ShoulderCenter], skeleton.Joints[JointType.ShoulderLeft], drawBrush);
            addLine(skeleton.Joints[JointType.ShoulderLeft], skeleton.Joints[JointType.ElbowLeft], drawBrush);
            addLine(skeleton.Joints[JointType.ElbowLeft], skeleton.Joints[JointType.WristLeft], drawBrush);
            addLine(skeleton.Joints[JointType.WristLeft], skeleton.Joints[JointType.HandLeft], drawBrush);

            // Right arm
            addLine(skeleton.Joints[JointType.ShoulderCenter], skeleton.Joints[JointType.ShoulderRight], drawBrush);
            addLine(skeleton.Joints[JointType.ShoulderRight], skeleton.Joints[JointType.ElbowRight], drawBrush);
            addLine(skeleton.Joints[JointType.ElbowRight], skeleton.Joints[JointType.WristRight], drawBrush);
            addLine(skeleton.Joints[JointType.WristRight], skeleton.Joints[JointType.HandRight], drawBrush);
        }

        void myKinect_SkeletonFrameReady(object sender, SkeletonFrameReadyEventArgs e)
        {
            string status = "";

            // find two trackedSkeletons
            List<Skeleton> trackedSkeletons = new List<Skeleton>();

            Skeleton[] skeletons = null;

            using (SkeletonFrame frame = e.OpenSkeletonFrame())
            {
                if (frame != null)
                {
                    skeletons = new Skeleton[frame.SkeletonArrayLength];
                    frame.CopySkeletonDataTo(skeletons);
                }
            }

            if (skeletons == null) return;

            foreach (Skeleton skeleton in skeletons)
            {
                if (skeleton.TrackingState == SkeletonTrackingState.Tracked)
                    trackedSkeletons.Add(skeleton);
            }

            // Remove the old skeleton drawings
            skeletonCanvas.Children.Clear();

            if (trackedSkeletons.Count < 2)
                status = string.Format("{0} person(s) present", trackedSkeletons.Count);
            else
            {
                drawSkeleton(trackedSkeletons[0], new SolidColorBrush(Colors.Red));
                drawSkeleton(trackedSkeletons[1], new SolidColorBrush(Colors.Blue));

                float distance = jointDistance(trackedSkeletons[0].Joints[JointType.Head],
                    trackedSkeletons[1].Joints[JointType.Head]);

                if (distance < 0.4f)
                {
                    status = "Steady on now";

                    if (oldDistance > 0.4f)
                    {
                        alarmPlayer.Play();
                    }
                }
                else
                {
                    status = "OK";
                }

                oldDistance = distance;
            }

            kissStatus.Text = status;
        }

        private float oldDistance = 100;

        private float jointDistance(Joint first, Joint second)
        {
            float dX = first.Position.X - second.Position.X;
            float dY = first.Position.Y - second.Position.Y;
            float dZ = first.Position.Z - second.Position.Z;

            return (float)Math.Sqrt((dX * dX) + (dY * dY) + (dZ * dZ));
        }
    }
}
