﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;

namespace MidiMan
{
    /// <summary>
    /// Interaction logic for MainWindow.xaml
    /// </summary>
    public partial class MainWindow : Window
    {
        public MainWindow()
        {
            InitializeComponent();
        }

        MIDIControl myMidi;

        private void setupMidi()
        {
            try
            {
                myMidi = new MIDIControl(0);
            }
            catch
            {
                MessageBox.Show("Midi setup failed", "Midi Control");
                Application.Current.Shutdown();
            }
        }

        int noOfKeys = 13;

        Brush lightKeyBrush = new SolidColorBrush(Colors.Green);
        Brush darkKeyBrush = new SolidColorBrush(Colors.DarkGreen);

        private void createKeyboard( )
        {
            double keyWidth = keyboardCanvas.Width / noOfKeys;
            for ( int i=0; i < noOfKeys; i++ ) 
            {
                // Make a new key
                TextBlock key = new TextBlock();

                // Set the key size
                key.Width = keyWidth;
                key.Height = keyWidth;

                // Make alternate keys different colors
                if (i % 2 == 0)
                    key.Background = lightKeyBrush;
                else
                    key.Background = darkKeyBrush;
                
                // Place the key on the keyboard
                keyboardCanvas.Children.Add(key);
                Canvas.SetTop(key,0);
                Canvas.SetLeft(key, i * keyWidth);

                // tag the key with the key number
                key.Tag = 60 + i; 

                // Connect the events
                key.MouseDown += new MouseButtonEventHandler(key_MouseDown);
                key.MouseUp += new MouseButtonEventHandler(key_MouseUp);
            }
        }

        void key_MouseUp(object sender, MouseButtonEventArgs e)
        {
            // Conver the sender into a TextBlock
            TextBlock key = (TextBlock)sender;

            // Extract the key number from the tag
            int keyNo = (int)key.Tag;

            myMidi.SendMIDIMessage(
                            0x80,        // midi command - note up on channel 0
                            keyNo,  // note value – 60 represents middle C on the keyboard
                            100);        // note velocity – how "hard" the note was released in the range 0 to 127
        }

        void key_MouseDown(object sender, MouseButtonEventArgs e)
        {
            // Convert the sender into a TextBlock
            TextBlock key = (TextBlock)sender;

            // Extract the key number from the tag
            int keyNo = (int)key.Tag;

            myMidi.SendMIDIMessage(
                            0x90,       // midi command - note down on channel 0
                            keyNo, // note value – 60 represents middle C on the keyboard
                            100);       // note velocity – how hard the note was hit in the range 0 to 127
        }


        private void Window_Loaded(object sender, RoutedEventArgs e)
        {
            setupMidi();
            createKeyboard();
        }

        private void note_MouseDown(object sender, MouseButtonEventArgs e)
        {
            myMidi.SendMIDIMessage(
                            0x90,  // midi command - note down on channel 0
                            60,    // note value – 60 represents middle C on the keyboard
                            100);  // note velocity – how hard the note was hit in the range 0 to 127
        }

        private void note_MouseUp(object sender, MouseButtonEventArgs e)
        {
            myMidi.SendMIDIMessage(
                            0x80,  // midi command - note up on channel 0
                            60,    // note value – 60 represents middle C on the keyboard
                            100);  // note velocity – how "hard" the note was released in the range 0 to 127
        }
    }
}
